/**
 * @brief FootyLine.h
 * @brief Footy̍ssPʂŊǗNXłB
 * @author Shinji Watanabe
 * @date Nov.09.2008
 */

#pragma once

#include "StaticVector.h"
#include "EmphasisWord.h"

// URLƂ̏
class CUrlInfo
{
public:
	CUrlInfo()
	{
		m_nStartPos = 0;
		m_nEndPos = 0;
	}
	
	// |WV܂܂邩`FbN
	inline bool IsInPosition(size_t nPosition)
	{
		return m_nStartPos <= nPosition &&
			nPosition < m_nEndPos;
	}
	inline size_t GetLength() const
	{
		return m_nEndPos - m_nStartPos;
	}

public:
	size_t m_nStartPos;					//!< URL̊Jnʒu0x[X
	size_t m_nEndPos;					//!< URL̏Iʒu0x[X
};

typedef std::vector<CUrlInfo>::iterator UrlIterator;

class CFootyLine
{
public:
	//! GUIɑR}h\
	struct EmpPos
	{
		size_t m_nPosition;				//!< R}ḧʒu
		bool m_bIsStart;				//!< truêƂAFς|Cg
		COLORREF m_Color;				//!< ݒ肷F
	};
	
	//! _v[`̖߂
	struct EthicInfo
	{
		size_t m_nEthicLine;			//!< _s
		size_t m_nEthicColumn;			//!< _ʒu
	};
	
	//! P̏
	struct WordInfo
	{
		size_t m_nBeginPos;				//!< Jnʒu(0x[X)
		size_t m_nEndPos;				//!< Iʒu(0x[X)
	};
	
	// RXgN^
	CFootyLine();
	
	// 񑀍IɌĂяo
	typedef CStaticVector<WordPt,sizeof(int)*8> TakeOver;
	bool SearchEmphasis(TakeOver *pBeforeBetween,LsWords *plsWords);
	bool FlushString(size_t nTabLen,size_t nColumn,int nMode);
	bool SetPrevLineInfo(std::list<CFootyLine>::iterator pPrevLine);
	inline void SetEmphasisChached(bool bChached){m_bEmphasisChached = bChached;}

	// 擾[`
	const wchar_t *GetLineData() const {return m_strLineData.c_str();}
	size_t GetLineLength() const {return m_strLineData.size();}
	inline size_t GetEthicLine() const { return m_nEthicLine; }
	inline size_t GetOffset() const {return m_nLineOffset;}
	inline size_t GetRealLineNum() const {return m_nRealLineNum;}
	inline CStaticVector<WordPt,sizeof(int)*8> *GetBetweenNext()
		{return &m_vecLineBetweenAfter;}
	inline std::vector<EmpPos> *GetColorInfo(){return &m_vecColorInfo;}
	inline std::vector<CUrlInfo> *GetUrlInfo(){return &m_vecUrlInfo;}
	inline std::vector<CUrlInfo> *GetMailInfo(){return &m_vecMailInfo;}
	inline bool EmphasisChached() const {return m_bEmphasisChached;}
	inline int GetLineIcons() const {return m_nLineIcons;}
	inline void SetLineIcons(int nLineIcons){m_nLineIcons = nLineIcons;}

	// ɌvZs[`
	EthicInfo CalcEthicLine(size_t nPos,size_t nColumn,size_t nTab,int nMode) const;
	size_t CalcRealPosition(size_t nEthicLine,size_t nEthicPos,size_t nColumn,size_t nTab,int nMode) const;
	bool IsGoNext(const wchar_t *pChar,size_t nPos,size_t nNowCol,size_t nColumns,int nMode) const;
	WordInfo GetWordInfo( size_t nPos, bool bIsBackMode = true ) const;
	size_t CalcAutoIndentPos( size_t nPos ) const;

	// ֘A
	enum CharSets
	{
		CHARSETS_ANSI_ALPHABET,				//!< ANSI(At@xbg)
		CHARSETS_ANSI_NUMBER,				//!< ANSI()
		CHARSETS_ANSI_SYMBOL,				//!< ANSI(A_[o[L)
		CHARSETS_ANSI_UNDER,				//!< ANSIA_[o[)
		CHARSETS_HIRAGANA,					//!< {(Ђ炪)
		CHARSETS_KATAKANA,					//!< {(J^Ji)
		CHARSETS_KATAKANA_HALF,				//!< {(pJ^Ji)
		CHARSETS_KANJI,						//!< {()
		CHARSETS_FULL_ALPHABET,				//!< SpAt@xbg
		CHARSETS_FULL_NUMBER,				//!< Sp
		CHARSETS_AINU_EXTENSION,			//!< ACkpg
		CHARSETS_IPA_EXTENSION,				//!< IPAg
		CHARSETS_SYMBOLS,					//!< L
		CHARSETS_JP_NOBASHI,				//!< {̂΂
		CHARSETS_KOREA,						//!< ؍
		CHARSETS_LATIN,						//!< e
		CHARSETS_BALTIC,					//!< og
		CHARSETS_ARABIC,					//!< ArA
		CHARSETS_HEBREW,					//!< wuC
		CHARSETS_GREEK,						//!< MV
		CHARSETS_CYRILLIC,					//!< L
		CHARSETS_VIETTNAMESE,				//!< xgi
		CHARSETS_THAI,						//!< ^C
		
		CHARSETS_UNKNOWN,					//!< s
	};

	static CharSets GetCharSets(wchar_t w);
	static CharSets GetCharSets(wchar_t w1,wchar_t w2);
	static bool IsDualChar(CharSets c);		/*{̉̕擾*/
	static inline bool IsDualChar(wchar_t w){return IsDualChar(GetCharSets(w));}
	static inline bool IsDualChar(wchar_t w1,wchar_t w2){return IsDualChar(GetCharSets(w1,w2));}
	static inline bool IsSurrogateLead(wchar_t w){return 0xD800 <= w && w <= 0xDBFF;}
	static inline bool IsSurrogateTail(wchar_t w){return 0xDC00 <= w && w <= 0xDFFF;}

private:
	// [`
	bool IsMatched(const wchar_t *pStr1,const wchar_t *pStr2,size_t nLen);
	
	// NbJu֘Ȁ
	bool FindMail(size_t start,size_t *pBegin,size_t *pEnd);
	bool FindURL(size_t start,size_t *pBegin,size_t *pEnd);
	bool IsMailChar(wchar_t wc);
	bool IsURLChar(wchar_t wc);

public:
	std::wstring m_strLineData;				//!< s̃f[^

private:
	std::vector<EmpPos> m_vecColorInfo;		//!< F
	TakeOver m_vecLineBetweenAfter;			//!< ̌Ɉps\
	std::vector<CUrlInfo> m_vecUrlInfo;		//!< URL
	std::vector<CUrlInfo> m_vecMailInfo;	//!< [
	size_t m_nEthicLine;					//!< _sH
	size_t m_nLineOffset;					//!< _sItZbgʒu
	size_t m_nRealLineNum;					//!< {̍sԍ
	bool m_bEmphasisChached;				//!< 񂪃LbVĂ
	int m_nLineIcons;						//!< sƂ̃ACR
};

// ^Cv̐錾
typedef std::list<CFootyLine>::iterator LinePt;
typedef std::list<CFootyLine>::size_type LineSize;
typedef std::list<CFootyLine> LsLines;

/*[EOF]*/
